# -*- coding: utf-8 -*-

import dataiku
from dataiku.core import flow
from dataiku.customrecipe import *
import numpy as np
from dataiku import pandasutils as pdu
import pandas as pd
import warnings
import os

import DoctorTaskSettingExtended


def configure_mltask(ml_task, ignored_column_type_list, metric, algorithm_name='', algorithm_setting_dict={}, ignored_column_list=[], sampling=True, custom_algorithm='', deployed_model_name=''):
    
    """ 
    Parameters
    ---------- 

    raw_ml_task: MLTask object

    ignored_column_type_list: list of string

    metric: string

    algorithm_name: string

    algorithm_setting_dict: dict

    ignored_column_list: list of string

    sampling: boolean

    custom_algorithm: string

    deployed_model_name: string


    Returns
    -------

    ml_task  
    """

    ml_setting = ml_task.get_settings()

    ml_setting_ext = DoctorTaskSettingExtended.DSSMLTaskSettingsExtended(ml_setting.client,
                                                                         ml_setting.project_key,
                                                                         ml_setting.analysis_id,
                                                                         ml_setting.mltask_id,
                                                                         ml_setting.mltask_settings)

    # FEATURE SETTINGS

    # disable rescaling on numerical features
    ml_setting_ext.set_mass_feature_setting(criteria_dict={'type': 'NUMERIC', 'role': 'INPUT'},
                                            new_setting_dict={'rescaling': 'NONE'})

    # reject all columns that are not numerical because after post-processing,
    # we can not retrieve their names for the rule extractor algorithm
    for column_type in ignored_column_type_list:

        ml_setting_ext.set_mass_feature_setting(criteria_dict={'type': column_type, 'role': 'INPUT'},
                                                new_setting_dict={'role': 'REJECT'})

    # remove columns in ignore_list
    for (feature_name, feature_setting) in ml_setting_ext.get_raw()['preprocessing']['per_feature'].items():

        if feature_name in ignored_column_list:
            feature_setting['role'] = 'REJECT'

    if not sampling:

        ml_setting_ext.get_raw()['splitParams']['ssdSelection'][
            'samplingMethod'] = 'FULL'

    # ALGORITHM SETTINGS

    if custom_algorithm:

        if not deployed_model_name:

            raise ValueError(
                'When using custom model, you need to define the model name.')

        ml_setting_ext.set_algorithm_list_enabled(algorithm_list=[])

        custom_model_dict = {
            'enabled': True,
            'code': "",
            'name': deployed_model_name
        }
        ml_setting_ext.get_raw()['modeling'][
            'custom_python'].append(custom_model_dict)
        ml_setting_ext.get_raw()['modeling']['custom_python'][0][
            'code'] = custom_algorithm  # custom_python is a list of custom model

    else:

        ml_setting_ext.set_algorithm_list_enabled(
            algorithm_list=[algorithm_name])
        ml_setting_ext.set_algorithm_setting(
            algorithm_name, algorithm_setting_dict)

    ml_setting_ext.set_metric(metric)
    ml_setting_ext.save()

    return ml_task


def deploy_mltask(deployed_model_name, ml_task, project, train_dataset):
    
    """
    Parameters
    ---------- 

    deployed_model_name: string

    ml_task: MLTask object

    project: string



    Returns
    -------


    """

    training_session = ml_task.start_train(session_name=deployed_model_name)
    ml_task.wait_train_complete()

    model_id_list = ml_task.get_trained_models_ids(
        training_session['sessionId'])
    # not a good idea if 2 models have the same name
    saved_model_dict = {model['name']: model['id']
                        for model in project.list_saved_models()}
    saved_deployed_model_name_list = saved_model_dict.keys()

    if deployed_model_name in saved_deployed_model_name_list:

        print 'Redeploy preprocessing model to flow'

        impacts_Downstream = ml_task.redeploy_to_flow(model_id=model_id_list[0],
                                                      saved_model_id=saved_model_dict[deployed_model_name])

        saved_model_id = saved_model_dict[deployed_model_name]

    else:

        print 'Deploy new preprocessing model to flow'
        saved_model_info_dict = ml_task.deploy_to_flow(model_id=model_id_list[0],
                                                       model_name=deployed_model_name,
                                                       train_dataset=train_dataset)

        saved_model_id = saved_model_info_dict['savedModelId']

    from dataiku.core import flow
    pf = flow.FLOW
    flow.FLOW = None
    model = dataiku.Model(saved_model_id)
    flow.FLOW = pf

    return model
