# -*- coding: utf-8 -*-

import dataiku
from dataiku import pandasutils as pdu
import pandas as pd
import numpy as np
from sklearn.base import BaseEstimator

import logging

logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO,  # avoid getting log from 3rd party module
                    format='RuleGeneration plugin %(levelname)s - %(message)s')


class RuleGeneratorRandomForestParams(BaseEstimator):

    def __init__(self, ml_type='clf', optimisation_metric='ROC_AUC', n_trees_list=[100], min_samples_leaf_list=[5], max_depth_list=[3]):

        self.ml_type = ml_type
        self.optimisation_metric = optimisation_metric
        self.param_grid = {'n_estimators': n_trees_list,
                           'min_samples_leaf': min_samples_leaf_list, 'max_depth': max_depth_list}
        self._performance_cache = []

    @property
    def ml_type(self):
        return self._ml_type

    @ml_type.setter
    def ml_type(self, value):
        if value not in ['clf', 'reg']:
            raise ValueError('ML type must be "clf" or "reg".')
        self._ml_type = value

    @property
    def optimisation_metric(self):
        return self._optimisation_metric

    @optimisation_metric.setter
    def optimisation_metric(self, value):
        self._optimisation_metric = value

    @property
    def n_estimators(self):
        return self._n_estimator

    @n_estimators.setter
    def n_estimators(self, value):
        if not all(map(lambda x: x < 0, value)):
            raise ValueError("The n_etimators list needs to be all positive")
        self._n_estimators = value

    @property
    def min_samples_leaf(self):
        return self._min_samples_leaf

    @min_samples_leaf.setter
    def min_samples_leaf(self, value):
        if not all(map(lambda x: x < 0, value)):
            raise ValueError(
                "The min_samples_leaf list needs to be all positive")
        self._min_samples_leaf = value

    @property
    def max_depth(self):
        return self._max_depth

    @max_depth.setter
    def max_depth(self, value):
        if not all(map(lambda x: x < 0, value)):
            raise ValueError("The max_depth list needs to be all positive")
        self._max_depth = value

    @property
    def clf(self):
        return self._clf

    @clf.setter
    def clf(self, value):

        ml_type, param_grid, optimisation_metric = value

        REGRESSION_METRICS = ['r2', 'neg_mean_absolute_error',
                              'neg_mean_squared_error', 'explained_variance']
        CLASSIF_METRICS = ['f1', 'roc_auc', 'accuracy', 'precision', 'recall']

        if ml_type == 'reg' and optimisation_metric not in REGRESSION_METRICS:
            raise ValueError('{0} is not valid. Possible regession metrics are: {1}'.format(
                optimisation_metric, REGRESSION_METRICS))

        if ml_type == 'clf' and optimisation_metric not in CLASSIF_METRICS:
            raise ValueError('{0} is not valid. Possible classification metrtics are: {1}'.format(
                optimisation_metric, CLASSIF_METRICS))

        if ml_type == 'clf':
            chosen_clf = RandomForestClassifier(random_state=1407, n_jobs=-1)
            logger.info(
                "The trees will be extracted from a RandomForestClassifier.")

        else:
            chosen_clf = RandomForestRegressor(random_state=1407, n_jobs=-1)
            logger.info(
                "The trees will be extracted from a RandomForestRegressor.")

        self._clf = GridSearchCV(estimator=chosen_clf,
                                 param_grid=param_grid,
                                 cv=2,
                                 scoring=optimisation_metric,
                                 verbose=1,
                                 n_jobs=-1)

    @property
    def performance_cache(self):
        return self._performance_cache

    def check(self):

        REGRESSION_METRICS = ['r2', 'neg_mean_absolute_error',
                              'neg_mean_squared_error', 'explained_variance']
        CLASSIF_METRICS = ['f1', 'roc_auc', 'accuracy', 'precision', 'recall']

        if self.ml_type == 'reg' and self.optimisation_metric not in REGRESSION_METRICS:
            raise ValueError('{0} is not valid. Possible regession metrics are: {1}'.format(
                self.optimisation_metric, REGRESSION_METRICS))

        if self.ml_type == 'clf' and self.optimisation_metric not in CLASSIF_METRICS:
            raise ValueError('{0} is not valid. Possible classification metrtics are: {1}'.format(
                self.optimisation_metric, CLASSIF_METRICS))
